function [fun, grad, parts] = JR_Poisson_Gaussian(x, s_image, ...
    pet_data, pet_operator, pet_adjoint, ...
    mri_data, mri_operator, mri_adjoint, mri_sigma, ...
    mu1, mu2)
% JR_Poisson_Gaussian
%   [fun, grad, parts] = JR_Poisson_Gaussian(x, s_image, ...
%    pet_data, pet_operator, pet_adjoint, ...
%   mri_data, mri_operator, mri_adjoint, mri_sigma, ...
%   mu1, mu2)
% computes the joint PET-MRI data term consisting of the negative 
% log-likelihoods for the Poisson and Gaussian distribution.
% 
% Input:  
%   x [vector]            
%       the PET-MRI image as a vector; PET first and then MRI
%
%   s_image [vector]
%       the size of the input as an image
%
%   pet_data [vector]
%       PET data
%
%   pet_operator [matrix or function handle]
%       maps input into the sinogram space; projection
%
%   pet_adjoint [matrix or function handle]
%       adjoint of the operator; backprojection
%
%   mri_data [vector]
%       MRI data; k-space
%
%   mri_operator [matrix or function handle]
%       maps input into the k-space; Fourier transform plus sampling
%
%   mri_adjoint [matrix or function handle]
%       adjoint of the operator; zero filling and inverse Fourier transform 
%
%   mri_sigma [scalar]
%       estimated standard deviation of the noise in k-space
%
%   mu1 [scalar; optional]
%       scaling factor in front of the PET data term; DEFAULT = 1
%
%   mu2 [scalar; optional]
%       scaling factor in front of the MRI data term; DEFAULT = 1
%
% Output:
%   fun [scalar]
%       function value
%
%   grad [vector]
%       gradient
%
%   parts [vector]
%       parts of this function
%
% See also: JR_PETMRI_JTV JR_PETMRI_LPLS JR_PETMRI_QPLS
%
% 2014-08-06 --------------------------------------------------------------
% Matthias J. Ehrhardt
% CMIC, University College London, UK 
% matthias.ehrhardt.11@ucl.ac.uk
% http://www.cs.ucl.ac.uk/staff/ehrhardt/software.html
%
% -------------------------------------------------------------------------
% Copyright 2014 University College London
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distributed on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
% -------------------------------------------------------------------------

    if numel(s_image) == 1; s_image = s_image * [1 1]; end;
    
    if nargin < 10
        mu1 = 1;
        mu2 = 1;
    end
    
    x_pet = x(1:end/2);
    x_mri = x(end/2+1:end);
    
    % data pet
    if mu1 > 0
        if nargout > 1
            [fun1, grad1] = JR_Poisson( ...
                x_pet, s_image, pet_data, pet_operator, pet_adjoint);
            grad1 = grad1 * mu1;
            
        else
            fun1 = JR_Poisson( ...
                x_pet, s_image, pet_data, pet_operator, pet_adjoint);
        end
        
        fun1 = fun1 * mu1;
    else
        fun1 = 0;
        grad1 = 0*x_pet;
    end
    
    % data mri
    if mu2 > 0
        res = mri_operator(x_mri) - mri_data;
        fun2 = 0.5 * sum(abs(res(:)).^2) / (mri_sigma^2) * mu2;
        
        if nargout > 1; grad2 = mri_adjoint(res) / (mri_sigma^2) * mu2; end;
    else
        fun2 = 0;
        grad2 = 0*x_mri;
    end
    
    % sum up
    if nargout > 1
        N = length(x);
        i_pet = 1:N/2;
        i_mri = N/2+1:N;
        
        grad = zeros(N,1);
        grad(i_pet) = grad1;
        grad(i_mri) = grad2;
    end
    
    if nargout > 2
        parts(1) = fun1;
        parts(2) = fun2;
    end
        
    fun = fun1 + fun2;
    
end